import { Attributes as GAttributes } from 'graphology-types';
import { XYPosition } from 'reactflow';
import { MultiGraph } from 'graphology';
import './utils';
import { GeneralDescription, InputNodeDimension, InputNodeType, InputNodeTypeTypes } from '../logic/general';
import { AllLogicTypes } from '../logic';
import { Handles, QueryElementTypes } from '../reactflow';
import { SchemaAttributeTypes } from '@graphpolaris/shared/lib/schema';

// export interface Attributes extends EntityNode | RelationNode | AttributeNode | FunctionNode | ModifierNode  {
// }

export interface NodeAttribute {
  handleData: QueryGraphEdgeHandle;
  // nodeCount: number;
  // summedNullAmount: number;
  // connectedRatio: number;
  // handles: string[];
}

export type NodeDefaults = {
  id?: string;
  schemaKey?: string;
  type: QueryElementTypes;
  width?: number;
  height?: number;
  attributes: NodeAttribute[];
  selected?: boolean;
};

/** Interface for the data in an entity node. */
export interface EntityData {
  name?: string;
  leftRelationHandleId?: QueryGraphEdgeHandle;
  rightRelationHandleId?: QueryGraphEdgeHandle;
  selected?: boolean;
  type: QueryElementTypes.Entity;
}

/** Interface for the data in an relation node. */
export interface RelationData {
  name?: string;
  collection: string;
  depth: { min: number; max: number };
  leftEntityHandleId?: QueryGraphEdgeHandle;
  rightEntityHandleId?: QueryGraphEdgeHandle;
  direction?: 'left' | 'right' | 'both';
  selected?: boolean;
  type: QueryElementTypes.Relation;
}

export interface LogicData {
  name: string;
  // inputType: InputNodeType;
  // logicType: AllLogicTypes;
  // key: string;
  logic: GeneralDescription<AllLogicTypes>;
  inputs: Record<string, InputNodeTypeTypes>; // name from InputNode -> InputNodeTypeTypes
  type: QueryElementTypes.Logic;
}

export type EntityNodeAttributes = XYPosition & NodeDefaults & EntityData;
export type RelationNodeAttributes = XYPosition & NodeDefaults & RelationData;
export type LogicNodeAttributes = XYPosition & NodeDefaults & LogicData;

export type QueryGraphNodes = EntityNodeAttributes | RelationNodeAttributes | LogicNodeAttributes;

export type QueryGraphEdgeAttribute = {
  attributeName: string;
  attributeType: SchemaAttributeTypes;
};

export type QueryGraphEdgeHandle = {
  nodeId: string;
  nodeName: string;
  nodeType: QueryElementTypes;
  handleType: Handles;
} & Partial<QueryGraphEdgeAttribute>;

export type QueryGraphEdges = {
  type: string;
  sourceHandleData: QueryGraphEdgeHandle;
  targetHandleData: QueryGraphEdgeHandle;
};

export type QueryGraphEdgesOpt = Partial<QueryGraphEdges>;

// export class QueryGraph extends Graph<QueryGraphNodes, GAttributes, GAttributes>; // is in utils.ts
